import 'package:flutter/material.dart';

/// Represents a verb. It consists of the [infinitive], the [translation] in the
/// language of the client app, the complete conjugation of the [indicative]
/// (present) and the language ([isoLanguage]) of the verb that is being practiced
class Verb with ChangeNotifier {
  String _infinitive;
  String _translation;
  List<String> _indicative;
  String _isoLanguage;

  String get isoLanguage => _isoLanguage;

  String get infinitive => _infinitive;

  set infinitive(String infinitive) {
    assert(infinitive.isNotEmpty, 'infinitive cannot be empty');
    _infinitive = infinitive;
    notifyListeners();
  }

  String get translation => _translation;

  set translation(String translation) {
    assert(translation.isNotEmpty, 'translation cannot be empty');
    _translation = translation;
    notifyListeners();
  }

  List<String> get indicative => _indicative;

  set indicative(List<String> indicative) {
    assert(indicative.isNotEmpty, 'indicative cannot be empty');
    assert(indicative.length == 6, 'indicative should contain six elements');
    _indicative = indicative;
    notifyListeners();
  }

  set isoLanguage(String isoLanguage) {
    assert(isoLanguage.isNotEmpty, 'isoLanguage cannot be empty');
    assert(isoLanguage.length == 2, 'isoLanguage should be two characters');
    _isoLanguage = isoLanguage;
    notifyListeners();
  }

  Verb(
      this._infinitive, this._translation, this._indicative, this._isoLanguage);
}
