import 'dart:collection';

import 'package:cramr_ui_lib/model/verb.dart';
import 'package:flutter/material.dart';

/// Model class for studying, testing and evaluating the verb. The model should
/// be initialised with the attributes of the [verb] that is being evaluated and
/// with the [pronouns] of the language. The instance also maintains a score
/// representing the amount of correct answers added by using the [add] method.
class VerbTestModel with ChangeNotifier {

  @override
  void dispose(){
    super.dispose();
    print('being disposed');
  }

  // verb under test
  late Verb _verb;

  // ignore: unnecessary_getters_setters
  set verb(Verb value) {
    _verb = value;
  }

  // Responses for evaluation
  List<String> _actualItems = ["", "", "", "", "", ""];

  // Pronouns off the bat
  List<String> _pronouns = [];

  // The actual score (right answers)
  int _score = 0;

  int get score => _score; //unmodifiable getter
  List<String> get pronouns => _pronouns; //unmodifiable getter

  String get targetLanguageIso => _verb.isoLanguage;


  // ignore: unnecessary_getters_setters
  Verb get verb => _verb; //unmodifiable getter

  UnmodifiableListView<String> get items => UnmodifiableListView(_actualItems);

  /// Initialise the model with the attributes of the [Verb] under test, as well
  /// as the [pronouns] of the language of the verb.
  /// Notifies listeners.
  void initialise(List<String> expected, List<String> pronouns,
      String targetLanguageIso, String infinitive, String translation) {
    this._pronouns = pronouns;
    this._verb = Verb(infinitive, translation, expected, targetLanguageIso);
    notifyListeners();
  }

  /// Resets the answers already given, for instance when giving another try.
  /// Notifies listeners.
  void reset() {
    _actualItems.clear();
    _actualItems = ["", "", "", "", "", ""];
    notifyListeners();
  }

  /// Adds the answer ([item]) to the list of answers on the zero-based position
  /// corresponding to the [person] of the conjugation
  void add(String item, int person) {
    _actualItems[person] = item.trim();
    _score = 0;
    for (int i = 0; i < _actualItems.length; i++) {
      if (_actualItems[i] != "" && _actualItems[i] == _verb.indicative[i]) {
        _score++;
      }
    }

    print("_items = $_actualItems");
    print("_score = $_score");
    notifyListeners();
  }

  /// Returns true if an answer has been submitted for all persons
  bool isComplete() {
    for (int i = 0; i < _actualItems.length; i++) {
      if ("" == _actualItems[i]) {
        return false;
      }
    }
    return true;
  }

}
