import 'dart:collection';
import 'package:cramr_ui_lib/model/verb.dart';


/// Repository singleton to be initialised with all the verbs that you want to
/// evaluate with the client app.
class VerbRepository {

  static VerbRepository _instance = VerbRepository._internal();

  VerbRepository._internal() {
    _instance = this;
  }

  /// Singleton factory method
  factory VerbRepository.instance() {
    return _instance;
  }

  HashMap<String, Verb> _verbs = HashMap<String, Verb>();
  List<String> _pronouns = [];

  set setVerbs(HashMap<String, Verb> value) {
    _verbs = value;
  }

  HashMap<String, Verb> get verbs => _verbs;
  List<String> get pronouns => _pronouns;

  /// The [verbs] with which you want to initialise the repository. The key for
  /// this HashMap should be the infinitive. Next to that, we initialise with
  /// the pronouns of the language that we are studying
  void initialise(HashMap<String, Verb> verbs, List<String> pronouns) {
    _verbs = verbs;
    _pronouns = pronouns;
  }
}
