import 'package:cramr_ui_lib/model/verb_test_model.dart';
import 'package:cramr_ui_lib/routes/verb_indicative_evaluate_route.dart';
import 'package:cramr_ui_lib/theme/custom_theme.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:provider/provider.dart';

/// The state of the form that allows you to show the evaluation of the
/// indicative of the [Verb]
class VerbIndicativeEvaluateFormState extends State<VerbIndicativeEvaluateRoute> {

  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      title: "Evaluatie",
      theme: CustomTheme.lightTheme,
      home: Scaffold(
        appBar: AppBar(
          actions: [
            _repeatIcon(context),
            _homeIcon(context),
          ],
          title: _titleCell(context),
        ),
        body: _centerWidget(context),
      ),
    );
  }

  IconButton _homeIcon(BuildContext context) {
    final VerbTestModel testModel = Provider.of<VerbTestModel>(context, listen: false);
    return IconButton(
        key: Key('menu_key'),
        onPressed: () {
          testModel.reset();
          Navigator.restorablePopAndPushNamed(context, '/verbSelectionRoute');
        },
        icon: Icon(Icons.home));
  }

  IconButton _repeatIcon(BuildContext context) {
    final VerbTestModel testModel = Provider.of<VerbTestModel>(context, listen: false);
    return IconButton(
        key: Key('repeat_key'),
        onPressed: () {
          testModel.reset();
          Navigator.popAndPushNamed(context, '/verbIndicativeStudyRoute');
        },
        icon: Icon(Icons.refresh));
  }

  Text _titleCell(BuildContext context) {
    final VerbTestModel testModel = Provider.of<VerbTestModel>(context, listen: false);
    return Text("Vervoeging ${testModel.verb.infinitive}");
  }

  Widget _centerWidget(BuildContext context) {
    return SafeArea(
      child: Scaffold(
        key: UniqueKey(),
        body: _bodyListView(context),
      ),
    );
  }

  Padding _scoreWidget(BuildContext context) {
    VerbTestModel model = context.watch<VerbTestModel>();
    return Padding(
      padding: const EdgeInsets.fromLTRB(20, 5, 20, 5),
      child: Card(
          elevation: 10,
          child: ListTile(
            title: Container(
              padding: EdgeInsets.fromLTRB(5, 5, 5, 5),
              child: Text(
                "Score:",
                style: CustomTheme.lightTheme.textTheme.headline1,
              ),
            ),
            trailing: Container(
              padding: EdgeInsets.fromLTRB(75, 5, 30, 5),
              child: Text(
                "${model.score}/6",
                style: CustomTheme.lightTheme.textTheme.bodyText1,
              ),
            ),
          )),
    );
  }


  Container _actualCell(BuildContext context, person) {
    VerbTestModel model = context.watch<VerbTestModel>();
    return Container(
      key: UniqueKey(),
      alignment: Alignment.center,
      child: ConstrainedBox(
        key: UniqueKey(),
        constraints: BoxConstraints(
          maxHeight: 25,
        ),
        child: _actualCellWidget(
            model.items[person], model.verb.indicative[person], person
        ),
      ),
    );
  }

  Widget _actualCellWidget(String actual, String expected, int person) {
    if (actual == expected) {
      return Row(
        key: UniqueKey(),
        children: [
          Text(
              "$actual",
              key: UniqueKey(),
              textAlign: TextAlign.right,
              style: TextStyle(
                color: Colors.green,
                fontSize: 25,
              )
          ),
        ],
      );
    } else {
      return Row(
        key: UniqueKey(),
        children: [
          Text(
              "$actual",
              key: UniqueKey(),
              textAlign: TextAlign.left,
              style: TextStyle(
                color: Colors.red,
                fontSize: 25,
              )
          ),
          FloatingActionButton(
              heroTag: 'wrong_${expected}_${person}',
              key: Key('wrong_${expected}_${person}'),
              backgroundColor: Colors.white,
              onPressed: () {_showCorrectAnswerDialog(expected);},
              child: Icon(
                Icons.help_outline_sharp,
                color: Colors.red,
              )
          ),
        ],
      );
    }
  }

  Future<void> _showCorrectAnswerDialog(String expected) async {
    return showDialog<void> (
        context: context,
        barrierDismissible: true,
        builder: (BuildContext context) {
          return AlertDialog(
            title: Text(
              "Juiste antwoord",
              key: UniqueKey(),
            ),
            content: Text("$expected",
                key: UniqueKey(),
                style: TextStyle(
                  color: Colors.green,
                  fontSize: 25,
                )),
            actions: [
              TextButton(
                  key: Key('got_it_button'),
                  onPressed: () {Navigator.of(context).pop();},
                  child: Text(
                    "Oké. Ik snap het.",
                    key: UniqueKey(),
                  ))
            ],
          );
        }
    );
  }

  Container _pronounCell(BuildContext context, person) {
    VerbTestModel model = context.watch<VerbTestModel>();
    return Container(
      key: UniqueKey(),
      alignment: Alignment.center,
      child: ConstrainedBox(
        key: UniqueKey(),
        constraints: BoxConstraints(
          maxHeight: 25,
        ),
        child: Text(
            "${_adaptedPronoun(model.targetLanguageIso, model.pronouns[person], model.verb.indicative[person])}",
            key: UniqueKey(),
            style: TextStyle(
              color: Colors.blue,
              fontSize: 25,
            )
        ),
      ),
    );
  }

  String _adaptedPronoun(String isoLang, String pronoun, String expected) {
    //TODO refactor with Strategy or the likes
    if ("fr" == isoLang) {
      if ("je" == pronoun &&
          (expected.substring(0, 1) == 'a' ||
              expected.substring(0, 1) == 'e' ||
              expected.substring(0, 1) == 'i' ||
              expected.substring(0, 1) == 'o' ||
              expected.substring(0, 1) == 'u')) {
        return "j\'";
      }
    }
    return pronoun;
  }

  ListView _bodyListView(BuildContext context) {
    return ListView(
      key: UniqueKey(),
      children: _bodyListTiles(context),
    );
  }

  List<Widget> _bodyListTiles(BuildContext context) {
    List<Widget> _widgets = [];

    _widgets.add(_scoreWidget(context));
    _widgets.add(Divider());

    List<Widget> _rows = [];
    for (int i = 0; i < 6; i++) {
      _rows.add(Card(
        key: UniqueKey(),
        elevation: 10,
        child: Padding(
          key: UniqueKey(),
          padding: const EdgeInsets.all(5.0),
          child: Row(
            key: UniqueKey(),
            children: [
              Padding(
                key: UniqueKey(),
                padding: const EdgeInsets.fromLTRB(20, 5, 5, 5),
                child: _pronounCell(context, i),
              ),
              Padding(
                key: UniqueKey(),
                padding: const EdgeInsets.fromLTRB(5, 5, 5, 5),
                child: _actualCell(context, i),
              ),
            ],
          ),
        ),
      ));
    }

    _widgets.add(
        Padding(
          key: UniqueKey(),
          padding: const EdgeInsets.fromLTRB(20, 0, 20, 0),
          child: Card(
            key: UniqueKey(),
            child: Column(
              key: UniqueKey(),
              children: _rows,
            ),
          ),
        ));

    return _widgets;
  }

}