import 'package:cramr_ui_lib/model/verb_test_model.dart';
import 'package:cramr_ui_lib/model/verb.dart';
import 'package:cramr_ui_lib/repo/verb_repository.dart';
import 'package:cramr_ui_lib/routes/verb_selection_route.dart';
import 'package:cramr_ui_lib/theme/custom_theme.dart';
import 'package:flutter/material.dart';
import 'package:provider/provider.dart';

/// State for the Route that allows you to select a verb
class VerbSelectionRouteState extends State<VerbSelectionRoute> {
  @override
  Widget build(BuildContext context) {
    return SafeArea(
      child: Scaffold(
        body: Center(
          key: Key('verbSelectionCenter'),
          child: ListView(
            key: Key('verbListView'),
            scrollDirection: Axis.vertical,
            controller: ScrollController(),
            physics: ScrollPhysics(
              parent: AlwaysScrollableScrollPhysics(),
            ),
            children: _repoButtons(context),
          ),
        ),
        // This trailing comma makes auto-formatting nicer for build methods.
      ),
    );
  }

  List<Widget> _repoButtons(BuildContext context) {
    List<Widget> result = [];

    var verbs = VerbRepository.instance().verbs;

    verbs.forEach((verbKey, value) {
      result.add(Card(
        key: Key('verbCard_${value}'),
        child: _verbButton(context, value),
        margin: EdgeInsets.fromLTRB(30, 15, 30, 15),
        elevation: 10.0,
      ));
    });

    return result;
  }

  ListTile _verbButton(BuildContext context, Verb verb) {
    return ListTile(
      key: Key("vervoeg ${verb.infinitive}"),
      visualDensity: VisualDensity.comfortable,
      contentPadding: EdgeInsets.fromLTRB(30, 10, 30, 30),
      dense: true,
      isThreeLine: true,
      title: Column(
        key: UniqueKey(),
        children: [
          Text(
            '${verb.infinitive}',
            key: UniqueKey(),
            style: CustomTheme.lightTheme.textTheme.headline1,
          ),
        ],
      ),
      subtitle: Column(
        key: UniqueKey(),
        children: [
          SizedBox(
            key: UniqueKey(),
            height: 40,
            child: Text(
              '${verb.translation}',
              key: UniqueKey(),
              style: CustomTheme.lightTheme.textTheme.bodyText2,
            ),
          ),
          _verbIcon(context, verb),
        ],
      ),
    );
  }

  Widget _verbIcon(BuildContext context, Verb verb) {
    return InkWell(
      key: UniqueKey(),
      child: CircleAvatar(
          key: UniqueKey() ,
          child:Icon(
              Icons.play_arrow,
              key: Key('startStudy_${verb.infinitive}')
          )
      ),
      onTap: () {
        VerbTestModel model = Provider.of<VerbTestModel>(context, listen: false);
        model.initialise(verb.indicative, VerbRepository.instance().pronouns, verb.isoLanguage,
            verb.infinitive, verb.translation);
        Navigator.restorablePopAndPushNamed(
            context, '/verbIndicativeStudyRoute');
      },
    );
  }
}
